//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	CLRseg
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Filtre passe-bas d'ordre 1 : 1/1+tau*p
//						Ce composant est similaire aucomposant standard CLR de XCOS
//
// Paramétrage		:	Constante de temps du filtre d'ordre 1
//						Valeur initiale du passe-bas à t=0
//
// Autres données	:	Le code objet est celui du composant CLR dans la Scilab
//						
//						
// Sorties			:	Signal filtré
//						
//						
//						
// Interface IHM	:	CLRseg.sci
// Code objet		;	SCI/modules/scicos_blocks/src/c/csslti4.c (Type 4) 
//
// Programme de test:	CLRseg.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2] = CLRseg(job,sciblk,e1)

    s1=[]; s2=[];
	
	Module="CLRseg";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
    
    select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
			
		
			x0=model.state
			rpar=model.rpar
		
			Tau=rpar(5);
			Vinit=rpar(6);
			
			ns=prod(size(x0));
			nin=1;nout=1;
			%scicos_context=%scicos_context; //copy the semi-global variable locally
			%scicos_context.s=%s //add s definition to the context

			Titre="Paramètres du filtre passe bas CLRseg"
			
			labels=["Constante de temps du filtre (s)";...
			        "Valeur initiale du filtre à t=0"];
					
			if getlanguage()=="en_US" then		

				Titre="CLRseg - Low pass filter parameters"
					
				labels=["Filter time constant (s)";...
			        "Filter initial value at t=0"];
										
			end
					
											
			while %t do
										
				[ok,Tau, Vinit,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",1),exprs)
									
				if ~ok then return; end	
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp); 
					else break;
				end
									
			end	
								
			// Mise en place du filtre passe-bas
			//----------------------------------
			num=1;
			den=1+Tau*%s;
								
			H=cont_frm(num,den)
			[A,B,C,D]=H(2:5);
			graphics.exprs=exprs;
			[ns1,ns1]=size(A)
			rpar=[matrix(A,ns1*ns1,1);
					matrix(B,ns1,1);
					matrix(C,ns1,1);
					D]
			if norm(D,1)<>0 then
						mmm=[%t %t];
			else
						mmm=[%f %t];
			end
			if or(model.dep_ut<>mmm) then
						model.dep_ut=mmm,
			end
			if ns1<=ns then
						x0=x0(1:ns1)
			else
						x0(ns1,1)=0
			end
					
			rpar=[rpar;Tau;Vinit]
					
			x0=Vinit*Tau;
			model.state=x0
			model.rpar=rpar
			sciblk.graphics=graphics;
			sciblk.model=model
					
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			Taudisplay=string(exprs(1))
							
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=left";
									"fillColor=#abcdee";
									"fontColor=blue";
									"displayedLabel=$\frac{1}{1 + \ " + Taudisplay + ".p\ }$"];
			sciblk.graphics.style = strcat(style_properties,";");
			
		
		case "define" then
			Vinit=0.;
			Tau=1;
			x0=Vinit*Tau;
			
			A=-1;
			B=1;
			C=1;
			D=0;

			model=scicos_model()
			
			model.sim=list("csslti4",4)		// Fonction Scilab en c dans modules\scicos_blocks
			
			model.in=1
			model.out=1
			model.state=x0
			model.rpar=[A(:);B(:);C(:);D(:);Tau;Vinit]
			model.blocktype="c"
			model.dep_ut=[%f %t]

			exprs=string([Tau;Vinit])
			sciblk = standard_define([4 2],model,exprs);
			
			style_properties = ["blockWithLabel";
                       "verticalLabelPosition=middle";
                       "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"fontSize=15";						
                        "displayedLabel=$\frac{1}{1+\tau.p\ }$"];
			sciblk.graphics.style = strcat(style_properties,";");
				
    end
	
endfunction
