//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	EdgeTrigger
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Détecteur de front d'un signal carré d'entrée
//
// Paramétrage		:	Trig=-1 pour détecter les fronts descendants
//						Trig=+1 pour détecter les fronts montants
//						Trig=0  pour détecter les fronts montants et descendants
//						Datation sur option
//
// Autres données	:	Typiquement, EdgeTrigger est utilisé pour détecter des fronts sur le signal d'entrée
//						Il et donc prévu pour être positionné derrière un relais (HYSERESIS Xcos)
//						
// Sorties			:	Signal de synchronisation
//						Possibilité de sortir ou non la datation de l'évènement sur un port de sortie régulier
//						
//						
// Interface IHM	:	EdgeTrigger.sci
// Code objet		;	EdgeTriggerC.c 
//
// Programme de test:	EdgeTrigger.zos, Busy.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=EdgeTrigger(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	typ=[];
	
	Module="EdgeTrigger";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détermime si mode simulation ou édition du composant par double clic
			
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end	

			labels=["Rising (1), Falling (-1), Both (0), Inhibit. (2)"...
			        "Datation : non (0) oui (1)"]
					
			meserrTrig="Erreur sur le détecteur de front : " + labels(1);
			meserrDate="Erreur sur la datation : " + labels(2);
										
			if getlanguage()=="en_US" then		
					
				labels=["Rising (1), Falling (-1), Both (0), Inhibit. (2)"...
			        "Dating : no (0) yes (1)"]
					
				meserrTrig="Error on the edge detector : " + labels(1);
				meserrDate="Dating error : " + labels(2);			
										
			end

			while %t do
			
				[ok,Trig,Datation,exprs]=scicos_getvalue("Set EDGETRIGGER block parameters",..
				labels,list("vec",-1,"vec",-1),exprs)
				
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if ~or(Trig==[-1:2]) then DefExp=[DefExp;meserrTrig]; end
							if ~or(Datation==[0:1]) then DefExp=[DefExp;meserrDate]; end
							
							if DefExp==[] then break; end							

							SegPalError(1, Module, DefExp);
				end		
				
			end
								
					graphics.exprs=exprs;
					model.rpar=[Trig;Datation]
					sciblk.graphics=graphics;
					
					outtyp=[]
					out=[]
					if (Datation==1) then
										 outtyp=1
										 out=[1, 1]
					end
					//Redessiner l'interface graphique en fonction de la valeur d'initialisation
					//--------------------------------------------------------------------------
					evtin=1;
					evtout=1;
					
					[model,graphics,ok] = set_io(model, graphics, list([model.in, model.in2],model.intyp), list(out,outtyp), evtin, evtout)

					sciblk.model = model;
					graphics.exprs = exprs;
					sciblk.graphics = graphics;					
										
					style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=Edge Trigger " + string(Trig)];
					sciblk.graphics.style = strcat(style_properties,";");
										

		
		case "define" then
			Trig=1			// Valeur d'initialisation sur front montant
			Datation=1		// Sortie datation du front
			exprs=[string(Trig);string(Datation)]
			
			model=scicos_model()
			model.sim=list("EdgeTriggerC",4)
			model.intyp=1
			model.in=1
			model.in2=1
			model.outtyp=1
			model.out=1
			model.out2=1
			model.evtin=1;
			model.evtout=1;
			model.rpar=[Trig;Datation]
			model.blocktype="c" 
			model.dep_ut=[%f %f]

			sciblk=standard_define([4 2],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=Edge Trigger " + string(Trig)];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
