//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	GeneBF
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2023
// Version			:	1.0
//
// Sujet			: 	Générateur de signaux basse fréquence
//
// Paramétrage		:	Sélection du mode de fonctionnement (modPer Echelon ou Sinus)
//						Boite Echelon (dw1, tdw1, dw2, tdw2)
//						Boite Sinus (tSinus, Asinus PSinus)
//						Boite Dirac (tDirac, ADirac, Pdirac, Ddirac, SDirac)
//
// Autres données	:	
//						
//						
// Sorties			:	Générateur opérationnel
//						
//						
//						
// Interface IHM	:	GeneBF.sci
// Code objet		;	GeneBFC.c 
//
// Programme de test:	GeneBF.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=GeneBF(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés	
	typ=[];
	
	Module="GeneBF";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;
			rpar=model.rpar;
						
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
			
			labelMode=["Mode de perturbation (0=Echelons 1=Sinusoïde)"];
			labelMode_en=["Disturbance mode (0=Steps 1=Sine wave)"];
					
			labelStep=["Amplitude échelon 1 : dw1";...
			        "Instant échelon 1 (s) : tdw1";...
					"Amplitude échelon 2 : dw2";...
					"Instant échelon 2 (s) : tdw2";]
			labelStep_en=["Step 1 amplitude : dw1";...
			        "Time step 1 (s) : tdw1";...
					"Step 2 amplitude : dw2";...
					"Time step 2 (s) : tdw2";]
					
			labelSinus=["Instant de départ (s) : tSinus";...
			        "Amplitude : ASinus";...
					"Période (s) : PSinus"]
			labelSinus_en=["Start time (s) : tSinus";...
			        "Amplitude : ASinus";...
					"Period (s) : PSinus"]
					
			labelDirac=["Instant de départ (s) : tDirac";...
			        "Amplitude : ADirac";...
					"Période (s) : PDirac";...
					"Durée Impulsion (s) : DDirac";...
					"Symérique (0 ou 1) : SDirac"];
			labelDirac_en=["Start time (s) tDirac";...
			        "Amplitude ADirac";...
					"Period (s) PDirac";...
					"Pulse Duration(s) DDirac";...
					"Symmetry (0 or 1) SDirac"];
					
			TitreEch=" - Echelons"
			TitreEch_en=" - Steps"
			
			TitreSin=" - Sinusoïde"
			TitreSin_en=" - Sinusoid" 

			meserr_tdw1="tdw1 doit être strictement positif";
			meserr_tdw1_en="tdw1 must be strictly positive";
			
			meserr_tdw2="tdw2 doit être strictement positif";
			meserr_tdw2_en="tdw2 must be strictly positive";
			
			meserrPSinus="La période de la fonction Sinus doit être strictement positive";
			meserrPSinus_en="The period of the Sine function must be strictly positive";
					
			messerrPDirac="La constante de temps PDirac doit être strictement positive";					
			messerrPDirac_en="The PDirac time constant must be strictly positive";					

			messerrDDirac="La durée du Dirac DDirac doit être strictement positive";					
			messerrDDirac_en="The duration of the Dirac DDirac must be strictly positive";					

			messerrSDirac="La valeur de SDirac est incorrecte (0 ou 1)";					
			messerrSDirac_en="The value of SDirac is incorrect (0 or 1))";					
			
					
			if getlanguage()=="en_US" then
			
				labelMode=labelMode_en
				labelStep=labelStep_en
				labelSinus=labelSinus_en
				labelDirac=labelDirac_en

				TitreEch=TitreEch_en
				TitreSin=TitreSin_en
				
				meserr_tdw1=meserr_tdw1_en				
				meserr_tdw2=meserr_tdw2_en				
				meserrPSinus=meserrPSinus_en
				messerrPDirac=messerrPDirac_en	
				messerrDDirac=messerrDDirac_en
				messerrSDirac=messerrSDirac_en
				
			end

			exprs1=exprs(1);
			exprs2=exprs(2:5)
			exprs3=exprs(6:8)
			exprs4=exprs(9:13)
								
			while %t do	
		
				[ok,modPer,exprs1]=scicos_getvalue("Set GeneBF",labelMode,...
												   list("vec",1),exprs1)
												   
				if ~ok then return; end

				[bdiag,DefExp]=CheckExprs(exprs1)				
				if bdiag && (modPer==0 || modPer==1) then break; end				
				if SegPalError(1, Module, labelMode(1))==2 then return; end
			end
			
			if modPer==0 then		
					// Mode Step
					//----------
					if ~isdef("tSinus") then tSinus=rpar(6); end
					if ~isdef("ASinus") then ASinus=rpar(7); end
					if ~isdef("PSinus") then PSinus=rpar(8); end
					

					while %t do			
						[ok,dw1,tdw1,dw2,tdw2,exprs2]=scicos_getvalue("Set GeneBF" + TitreEch,labelStep,...
												   list("vec",1,"vec",1,"vec",1,"vec",1),exprs2)
						if ~ok then return; end
						
						[bdiag,DefExp]=CheckExprs(exprs2)	
						if ~bdiag then SegPalError(2, Module, DefExp)	
						else
							DefExp=[]
							if tdw1<=0 then DefExp=[DefExp;meserr_tdw1]; end
							if tdw2<=0 then DefExp=[DefExp;meserr_tdw2]; end

							if DefExp==[] then break; end
														
							SegPalError(1, Module,DefExp);
						end
					end
			else
				// Mode Sinus
				//-----------
				if ~isdef("dw1") then dw1=rpar(2); end
				if ~isdef("tdw1") then tdw1=rpar(3); end
				if ~isdef("dw2") then dw2=rpar(4); end
				if ~isdef("tdw2") then tdw2=rpar(5); end
								
					while %t do			
						[ok,tSinus,ASinus,PSinus,exprs3]=scicos_getvalue("Set GeneBF" + TitreSin,labelSinus,...
												   list("vec",1,"vec",1,"vec",1),exprs3)
						if ~ok then return; end
						
						[bdiag,DefExp]=CheckExprs(exprs3)
						if ~bdiag then SegPalError(2, Module, DefExp)
						else
							DefExp=[]
							if PSinus<=0 then DefExp=[DefExp;meserrPSinus]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
						end
					end		
			end

			// Dirac
			//-------
			while %t do	
						[ok,tDirac,ADirac,PDirac,DDirac,SDirac,exprs4]=scicos_getvalue("Set GeneBF" + " - Dirac",labelDirac,...
												   list("vec",1,"vec",1,"vec",1,"vec",1,"vec",1),exprs4)
						
						if ~ok then return; end

				
						[bdiag,DefExp]=CheckExprs(exprs4)
						if ~bdiag then SegPalError(2, Module, DefExp)
						else
							DefExp=[]
							if PDirac<=0 then DefExp=[DefExp;messerrPDirac]; end
							if DDirac<=0 then DefExp=[DefExp;messerrDDirac]; end
							if PDirac<=DDirac then DefExp=[DefExp;"PDirac > DDirac"]; end
							if ~or(SDirac==[0;1]) then DefExp=[DefExp;messerrSDirac]; end
							
							if DefExp==[] then break; end
							
							SegPalError(1, Module, DefExp);
						end
						
			end
			
			rpar=[modPer;dw1;tdw1;dw2;tdw2;tSinus;ASinus;PSinus;tDirac;ADirac;PDirac;DDirac;SDirac];	
			exprs=[exprs1;exprs2;exprs3;exprs4]	

			graphics.exprs=exprs;
			model.rpar=rpar;	
					
			sciblk.graphics=graphics;
			sciblk.model=model
					
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=blue";
						"fontColor=black";
						"displayedLabel=GeneBF"];
			sciblk.graphics.style = strcat(style_properties,";");
														
		case "define" then
			// Mode de perturbation (0=Echelons 1=SinuSync)			
			//---------------------------------------------			
			modPer=0;			
			
			// Générateur Echelons			
			//---------------------			
			dw1=0.05;			// Amplitude échelon 1
			tdw1=9999;			// Instant échelon 1 (s)
			
			dw2=-dw1;			// Amplitude échelon 2
			tdw2=9999;			// Instant échelon 2 (s)
			
			// Générateur SinuSync			
			//--------------------			
			tSinus=9999;			// Instant de départ (s)
			ASinus=0.1;				// Amplitude
			PSinus=9999;			// Période (s)
			
			// Générateur Peigne de Dirac DiracSync			
			//-------------------------------------			
			tDirac=9999;		// Instant de départ (s)
			ADirac=0;			// Amplitude
			PDirac=20;			// Période (s)
			DDirac=0.1;			// Durée du Dirac (s)
			SDirac=1;			// Symétrie
			
			model=scicos_model()
			model.sim=list("GeneBFC",4)
			model.in=[];model.in2=[];
			model.out=1;model.out2=[1]
			
			rpar=[modPer;dw1;tdw1;dw2;tdw2;tSinus;ASinus;PSinus;tDirac;ADirac;PDirac;DDirac;SDirac];	
			exprs=[sci2exp(modPer); ..
						sci2exp(dw1);sci2exp(tdw1);sci2exp(dw2);sci2exp(tdw2); ..
						sci2exp(tSinus);sci2exp(ASinus);sci2exp(PSinus); ..
						sci2exp(tDirac);sci2exp(ADirac);sci2exp(PDirac);sci2exp(DDirac);sci2exp(SDirac)];	

			model.blocktype="c"
			model.dep_ut=[%f %t]
			model.rpar=rpar;	
			
			sciblk=standard_define([3. 2.],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=blue";
						"fontColor=black";
						"displayedLabel=GeneBF"];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction


