//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Hysteresis_H
// Auteur			:	PERRICHON Pierre
// Date				:	02/09/2021
// Version			:	1.0
//
// Sujet			: 	Hysteresis vectorié n lignes x 1 colonne
//
// Paramétrage		:	in1 size(m,1) type 1 - enable size(n,1) type 5 -  in2 size(m,1) 
//						out size n,1) type 1
//						m=1 type scalaire
//						m>1 on doit alors avoir m=n 
//
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	Hysteresis programmé sur chacune des entrées
//						
//						
// Interface IHM	:	Hysteresis_H.sci
// Code objet		;	Hysteresis_HC.c 
//
// Programme de test:	Hysteresis_H.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Hysteresis_H(job, sciblk, e1)

	s1= []; s2 = [];	// arguments requis mais non utilisés 
	
	Module="Hysteresis_H";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
  
    select job
	
    case "set" then
	
		graphics=sciblk.graphics;
		exprs=graphics.exprs;
		model=sciblk.model;
		
		// Analyse préliminaire des expressions
		//-------------------------------------
		[bdiag,DefExp]=CheckExprs(exprs) 		
		if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
		end
		
		labels=["switch on at";...
			        "switch off at";...
					"output when on";...
					"output when off"]
					
		meserrLimite="Switch on value must be larger than switch off value" ;

        while %t do
		
			
			[ok,high_lim,low_lim,out_high,out_low,exprs]=scicos_getvalue("Set Hysteresis_H",labels,...
												   list("vec",1,"vec",1,"vec",1,"vec", 1),exprs)
			
			
            if ~ok then return; end
			
			//Analyse du paramétrage
			//----------------------			
			[bdiag,DefExp]=CheckExprs(exprs)
			if ~bdiag then SegPalError(2, Module, DefExp)
				else
							DefExp=[]
							if low_lim>high_lim then 
													DefExp=[DefExp;meserrLimite];
							end 
							
							if DefExp==[] then break; end
							
							SegPalError(1, Module, DefExp);
			end
			
			
        end
		
        graphics.exprs=exprs;
        model.rpar=[high_lim,low_lim,out_high,out_low]'
        sciblk.graphics=graphics;
		sciblk.model=model
			   
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=HYST_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";")	
			   
		
    case "define" then
        rpar=[1;0;1;0]

        model=scicos_model()
        model.sim=list("Hysteresis_HC",4)
        model.in=-1
        model.out=-1
        model.rpar=rpar
        model.dep_ut=[%t %f]

        exprs=[string(rpar)]
        gr_i=[]
        sciblk=standard_define([3 2],model,exprs,gr_i)
		
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=HYST_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";")	
				
    end
endfunction
