//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Interpln_H
// Auteur			:	PERRICHON Pierre
// Date				:	02/09/2021
// Version			:	1.0
//
// Sujet			: 	Interplolation linéaire vectorisée sur une loi XY à absisses X croissantes
//					
//
// Paramétrage		:	in :  size(-1,1) type 1
//						out :  size(-1,1) type 1, idem in
//						Ci-dessous X et Y sont des matrices lignes de dimension mx1
//
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	
//						
//						
// Interface IHM	:	Interpln_H.sci
// Code objet		;	Interpln_HC.c 
//
// Programme de test:	Interpln_H.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Interpln_H(job, sciblk, e1)

	s1= []; s2 = [];	// arguments requis mais non utilisés 
	
	Module="Interpln_H";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic

    select job

    case "set" then
	
		graphics=sciblk.graphics;
		exprs=graphics.exprs;
		model=sciblk.model;

		// Analyse préliminaire des expressions
		//-------------------------------------
		[bdiag,DefExp]=CheckExprs(exprs) 		
		if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
		end
		
		labels=["X coord.";"Y coord."]
		
		meserr1="X and Y must have the same size";	
		meserr2="X must be strictly increasing";
				
        while %t do
		
            [ok,X,Y,exprs]=scicos_getvalue("Set Interpolation block parameters",..
            labels,list("vec",-1,"vec",-1),exprs)
			
            if ~ok then return; end
			
			//Analyse du paramétrage
			//----------------------			
			[bdiag,DefExp]=CheckExprs(exprs)
			if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if size(X,"*") <> size(Y,"*") then DefExp=[DefExp;meserr1]; end
							if min(X(2:$)-X(1:$-1)) <=0 then DefExp=[DefExp;meserr2]; end
							
							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
			end				
			
		end
			
        graphics.exprs=exprs;			
		model.opar=list(X,Y)		
		sciblk.graphics=graphics;
		sciblk.model=model
			
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=Interpln_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";")	
			
 				
    case "define" then
        X=[0;1]
        Y=[0;1]
		
		opar=list(X,Y)

        model=scicos_model();
        model.sim=list("Interpln_HC",4)
		
        model.in=-1; model.in2=1; model.intyp=1
        model.out=-1; model.out2=1; model.outtyp=1
        model.opar=opar

        model.dep_ut=[%t %f]
		
        exprs=[strcat(sci2exp(X));strcat(sci2exp(Y))]
        gr_i=[]
        sciblk=standard_define([3 2],model,exprs,gr_i)
		
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=Interpln_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";");	
		
    end
	
endfunction
