//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Memo
// Auteur			:	PERRICHON Pierre
// Date				:	30/08/2018
// Version			:	1.0
//
// Sujet			: 	Entrée retardée
//
// Paramétrage		:	Valeur initiale de la mémoire
//						
//						
//					
//
// Autres données	:	
//						
//						
// Sorties			:	Entrée retardée
//						
//
// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	26/08/2021
// Version			:	1.1
//
// Description		!	Vectorisation (prise en compte de la taille du dimensionnement de la mémoire)
//						
//						

// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	17/08/2022
// Version			:	1.2
//
// Description		!	Modification de l'état StateUpdate pour éviter un glitch à t=0	(voir fichier C)					
//
// Interface IHM	:	Memo.sci
// Code objet		;	MemoC.c 
//
// Programme de test:	Memo.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Memo(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	
	Module="Memo";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
	
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end	

			labels="Initial condition(s)"
			
			meserr1="Initial condition must be a (m x 1) vector";
			meserr2="Initial condition empty";
						
			while %t do
			
				[ok,x0,exprs]=scicos_getvalue("Set memory block parameters Memo",..
				labels,list("vec",-1),exprs)
				
				if ~ok then return; end
				
				[out,x]=size(x0)
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if x<>1 then DefExp=[DefExp;meserr1]; end
							if out==0 then DefExp=[DefExp;meserr2]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
								
			end	
				
			in=out
			model.evtin=[] //Rajout PPe
				
			graphics.exprs=exprs;
			model.state=x0
			model.rpar=x0;
			model.in=in;
			model.out=out
			sciblk.graphics=graphics;
			sciblk.model=model
					
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"fontSize=11";						
						"displayedLabel=Mémo <br>"   + string(exprs(1))];
			sciblk.graphics.style = strcat(style_properties,";");
							
		case "define" then
			x0=0			// Valeur d'initialisation
			in=1
			exprs=[string(x0)]
			model=scicos_model()
			model.sim=list("MemoC",4)
			model.in=in
			model.out=in
			model.evtin=[] //1	// Rajout PPe
			model.state=x0
			model.rpar=x0
			model.blocktype="c" //"m"
			model.dep_ut=[%f %f]

			sciblk=standard_define([2.5 2.5],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"fontSize=11";						
						"displayedLabel=Mémo <br>"   + string(x0)];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
