//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	PSB
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Filtre passe-bas d'ordre 1 : 1/1+tau*p avec réinitialisation et saturations externes
//						
//
// Paramétrage		:	Constante de temps du filtre d'ordre 1 (s)
//						Valeur initiale du passe-bas à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Signal filtré
//						
//						
//						
// Interface IHM	:	PSB.sci
// Code objet		;	PSBC.c 
//
// Programme de test:	PSB.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=PSB(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
  
	Module="PSB";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
  
  
	select job 
 
		case "set" then 
	
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;	

			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end

			Titre="Paramètres du filtre passe-bas PSB"
			
			labels=["Constante de temps du filtre passe-bas (s)";...
			        "Valeur initiale du filtre passe-bas à t=0";...
					"Avec réinitialisation programmable (1:oui - 0:non)";...
					"Avec saturation dynamique (1:oui - 0:non)"];
					
			meserrTau="La constante de temps du filtre passe-bas doit être strictement positive"
			meserrReinit="La valeur de la réinitialisation programmable est incorrecte"
			meserrSature="La valeur de la saturation dynamique est incorrecte"
			
			if getlanguage()=="en_US" then		

				Titre="PSB Low Pass Filter Parameters"

				labels=["Low-pass filter time constant (s)";...
			        "Initial value of the low-pass filter at t=0";...
					"With programmable reset (1:yes - 0:no)";...
					"With dynamic saturation (1:yes - 0:no)"];
					
				meserrTau="The time constant of the low-pass filter must be strictly positive"
				meserrReinit="The programmable reset value is incorrect"
				meserrSature="The dynamic saturation value is incorrect"

			end			
				
			while %t do
			
				[ok,Tau,x0,reinit,sature,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",1,"vec",1,"vec",1),exprs)
					
				if ~ok then return; end
					
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tau<=0 then DefExp=[DefExp;meserrTau]; end
							if ~or(reinit==[0 1]) then DefExp=[DefExp;meserrReinit]; end
							if ~or(sature==[0 1]) then DefExp=[DefExp;meserrSature]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
			
			end	
				
			n = size(x0,'*');
			out = [n, 1];
			ot = 1;
			evtout = []; //aucun port de sortie d'activation
					
			it = 1;
			in = [n, 1];
			graphics.in_label=[""];
			if(reinit + sature) then graphics.in_label=["in"];end;
			evtin = [];
										
			// Traitement de la réinitialisation
			//----------------------------------							
			if reinit~=0 then
							reinit=1;					
							// Rajout d'1 entrée régulière de dimension n x 1 
							//-----------------------------------------------
							it=[it, 1];
							in=[in; n,1];
							graphics.in_label=[graphics.in_label, "init"];

							// Un port d'entrée d'activation
							//------------------------------
							evtin = 1; 							
			end
					
			//Traitement de la saturation
			//---------------------------
			if sature~=0 then
						sature=1;
						// Rajout de 2 entrées régulières de dimension n x 1
						//---------------------------------------------------
						it=[it, 1, 1];
						in=[in; n,1; n, 1];
						graphics.in_label=[graphics.in_label,"UL","LL"];
			end
					
			//Redessiner l'interface graphique en fonction de la valeur d'initialisation
			//--------------------------------------------------------------------------
			[model,graphics,ok] = set_io(model, graphics, list(in,it), list(out, ot), evtin, evtout)				
				
			model.state = x0;			// Valeur initiale de l'intégrateur
			model.rpar=[Tau];			// Constante de temps de l'intégrateur
			model.ipar=[reinit;sature]							
							
			sciblk.model = model;
			graphics.exprs = exprs;
			sciblk.graphics = graphics;
					
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			Taudisplay=string(exprs(1))+ "."
			if strcmp(Taudisplay,"1.")==0 then Taudisplay="";end;
							
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=left";
									"fillColor=#abcdee";
									"fontColor=blue";
									"displayedLabel=$\frac{1}{1 + \ " + Taudisplay + "p\ }$"];
			sciblk.graphics.style = strcat(style_properties,";");

			
		case "define" then 

			Tau=1;		// Constante de temps du filtre passe-bas (s)
			x0=0;		// Valeur initiale nulle du filtre passe-bas			
			reinit=1;	// Réinitialisation du filtre passe-bas			
			sature=1;	// Saturation dynamique
			
			model = scicos_model();
			
			model.sim=list("PSBC",4)			// Pour appel fonction "c" PSBC
						
			model.intyp = [1;1;1;1];
			model.in = [1;1;1;1]; model.in2 = [1;1;1;1]; 		//4 entrées scalaire

			model.outtyp = 1;
			model.out = 1; model.out2 = 1;		//une sortie scalaire

			model.evtin = 1 					// 1 port d'entrée d'activation
			
			model.state = x0;
			
			model.ipar=[reinit;sature]
			model.rpar=[Tau]
			model.blocktype="c"
			
			model.dep_ut = [%t %t];
			
			exprs = [sci2exp(Tau);sci2exp(x0);sci2exp(reinit);sci2exp(sature)];
			
			sciblk = standard_define([3 3],model,exprs);
			
			style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=15";
                    "displayedLabel=$\frac{1}{1+\tau.p\ }$";
					"fillColor=#abcdee";
					"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
					
						
	end
  
endfunction