//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Pow
// Auteur			:	PERRICHON Pierre
// Date				:	01/05/2014
// Version			:	1.0
//
// Sujet			: 	Elévation à la puissance
//
// Paramétrage		:	Valeur de l'exposant
//						
//						
//						
//
// Autres données	:	...Introduit pour afficher l'exposant dans la boite
//						Bug Scilab 5.5.2 dans le composant POWBLK et donc correctif avec ce composant
//						
// Sorties			:	Entrée élevée à la puissance
//						
//						
//						
// Interface IHM	:	Pow.sci
// Code objet		;	ExponentC.c 
//
// Programme de test:	Pow.zos - Pow2.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2] = Pow(job,sciblk,e1)
    
    s1=[];
    s2=[];

	Module="Pow";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic

	Titre="Pow : Elévation à la puissance u^a"
	labels=["Valeur de l''exposant"]
	
	Titre_en="Pow : Raising to the power u^a"
	labels_en="Exponent value"
	
	if getlanguage()=="en_US" then
		Titre=Titre_en
		labels=labels_en
	end
	
    select job
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end			
				
			
			while %t do
					[ok,a,exprs]=scicos_getvalue(Titre,labels, ...
								list("vec",1),exprs)
			
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							
							//... Pas de source d'erreur à tester

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
			end
								
			graphics.exprs=exprs
			model.rpar=a;

			sciblk.graphics=graphics;
			sciblk.model=model
			
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			a_display="{" + string(exprs(1)) + "}"
					
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=left";
									"fillColor=#abcdee";
									"fontColor=blue";
									"displayedLabel=$u^" + a_display + "$"];
									
			sciblk.graphics.style = strcat(style_properties,";");
			
		
		case "define" then
			a=1.5
			model=scicos_model()
			model.sim=list("ExponentC", 4);
			model.in=1
			model.out=1
			model.rpar=a
			model.blocktype="c"
			model.dep_ut=[%t %f]

			exprs=string(a)
			sciblk=standard_define([2 2],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=$u^a$"];
			sciblk.graphics.style = strcat(style_properties,";");
		
	end
	
endfunction
