//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Rampe
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Rampe intégrateur avec réinitialisation et saturations externes
//						
//
// Paramétrage		:	Constante de temps du filtre d'ordre 1 (s)
//						Valeur initiale du passe-bas à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Rampe avec réinitialisation et saturations appliquées au signal d'entrée
//						
//						
//						
// Interface IHM	:	Rampe.sci
// Code objet		;	RampeC.c 
//
// Programme de test:	Rampe.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Rampe(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	
	Module="Rampe";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
  
	select job 
  
		case "set" then 
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
			
			Titre="Paramètres de la boîte Ramp"
						
			labels=["Constante de temps (s)";...
			        "Valeur initiale de la rampe à t=0";...
					"Avec réinitialisation programmable (1:oui - 0:non)";...
					"Avec saturation dynamique (1:oui - 0:non)"];
					
			meserr1="La constante de temps doit être positive ou nulle";	

			if getlanguage()=="en_US" then		
				Titre="Ramp Box Settings"
						
				labels=["Time constant (s)";...
			        "Initial value of the ramp at t=0";...
					"With programmable reset (1:yes - 0:no)";...
					"With dynamic saturation (1:yes - 0:no)"];
					
				meserr1="The time constant must be positive or zero";	
			end
					
			while %t do
			
				[ok,Tau,x0,reinit,sature,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",1,"vec",1,"vec",1),exprs)
					
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tau<0 then DefExp=[DefExp;meserr1]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
						
			end	
				
			n = size(x0,'*');
			out = [n, 1];
			ot = 1;
			evtout = []; //aucun port de sortie d'activation
					
			it = 1;
			in = [n, 1];
			graphics.in_label=[""];
			if(reinit + sature) then graphics.in_label=["in"];end;
			evtin = [];
										
			// Traitement de la réinitialisation
			//----------------------------------							
			if reinit~=0 then
							reinit=1;					
							// Rajout d'1 entrée régulière de dimension n x 1 
							//-----------------------------------------------
							it=[it, 1];
							in=[in; n,1];
							graphics.in_label=[graphics.in_label, "init"];

							// Un port d'entrée d'activation
							//------------------------------
							evtin = 1; 							
			end
					
			//Traitement de la saturation
			//---------------------------
			if sature~=0 then
						sature=1;
						// Rajout de 2 entrées régulières de dimension n x 1
						//---------------------------------------------------
						it=[it, 1, 1];
						in=[in; n,1; n, 1];
						graphics.in_label=[graphics.in_label,"UL","LL"];
			end
					
			//Redessiner l'interface graphique en fonction de la valeur d'initialisation
			//--------------------------------------------------------------------------
			[model,graphics,ok] = set_io(model, graphics, list(in,it), list(out, ot), evtin, evtout)
				
			// Terminé ==> transfert des structures et validation
			//---------------------------------------------------
			model.state = x0;			// Valeur initiale du filtre passe-bas
			model.rpar=[Tau];			// Constante de temps du filtre passe-bas
			model.ipar=[reinit;sature]							
							
			sciblk.model = model;
			graphics.exprs = exprs;
			sciblk.graphics = graphics;
							
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			Taudisplay=string(exprs(1))
							
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=middle";
									"fontColor=blue";
									"fontSize=11";
									"displayedLabel=Ramp :<br>" +  string(Taudisplay) + " (s)";
									"fillColor=#abcdee";
									"shadow=true"];									
			sciblk.graphics.style = strcat(style_properties,";");
											
	
		case "define" then 
	
			Tau=120;	// Constante de temps de la rampe (s)
			x0=0;		// Valeur initiale nulle du filtre passe-bas			
			reinit=1;	// Réinitialisation de la rampe		
			sature=1;	// Saturation dynamique
			
			model = scicos_model();
			
			model.sim=list("RampeC",4)			// Pour appel fonction "c" Rampe
						
			model.intyp = [1;1;1;1];
			model.in = [1;1;1;1]; model.in2 = [1;1;1;1]; 		//4 entrées scalaire

			model.outtyp = 1;
			model.out = 1; model.out2 = 1;		//une sortie scalaire

			model.evtin = 1 					// 1 port d'entrée d'activation
			
			model.state = x0;
			
			model.ipar=[reinit;sature]
			model.rpar=[Tau]
			model.blocktype="c"
			
			model.dep_ut = [%t %t];
			
			exprs = [sci2exp(Tau);sci2exp(x0);sci2exp(reinit);sci2exp(sature)];
			
			sciblk = standard_define([3 3],model,exprs);
			
			Taudisplay=string(exprs(1))		
			
			style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=Ramp :<br>" +  string(Taudisplay) + " (s)";
					"fillColor=#abcdee";
					"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
					
	end
  
endfunction