//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Relais_H
// Auteur			:	PERRICHON Pierre
// Date				:	02/09/2021
// Version			:	1.0
//
// Sujet			: 	Relais en parallèle sur un signal d'entrée 1x1
//
// Paramétrage		:	in :  size(1,1) type 1
//						Switch on : tabeau de seuils de dimention nx1
//						Switch off : tabeau de seuils de dimention nx1
//						Choix de polarité si dépassement de seuil haut dimension 1x1
//						Choix de polarité si dépassement de seuil bas dimension 1x1
//						out size (n,1) type 1
//						Possibilité d'échantillonnner la boite de relayage
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	Hysteresis programmé sur chacune des entrées
//						
//						
// Interface IHM	:	Relais_H.sci
// Code objet		;	Relais_HC.c 
//
// Programme de test:	Relais_H.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Relais_H(job, sciblk, e1)

	s1= []; s2 = [];	// arguments requis mais non utilisés 
	
	Module="Relais_H";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
  
    select job
	
    case "set" then
	
		graphics=sciblk.graphics;
		exprs=graphics.exprs;
		model=sciblk.model;
		
		// Analyse préliminaire des expressions
		//-------------------------------------
		[bdiag,DefExp]=CheckExprs(exprs) 		
		if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
		end
		
        labels=["switch on at";...
				"switch off at";...
				"output when on";...
				"output when off";...
				"Sampling (0=No 1=Yes)"];
				
		meserr1="Switch on and off must be of the same size";
		meserr2="Switch on value must be larger than switch off value"

		Titre="Paramètres de la boîte RELAIS_H"

		if getlanguage()=="en_US" then 

			Titre="RELAIS_H box parameters"	
			
		end
					
        while %t do
		
            [ok,high_lim,low_lim,out_high,out_low,Sampling,exprs]=scicos_getvalue(Titre,labels,...
            list("vec",-1,"vec",-1,"vec",1,"vec",1,"vec",1),exprs)
			
			if ~ok then return; end
			
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							
							nH=size(high_lim,"*")
							nL=size(low_lim,"*")
			
							if nH<>nL then DefExp=[DefExp;meserr1]; end
							if low_lim>high_lim then DefExp=[DefExp;meserr2]; end
							
							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end	

			end	
			
			
		n=nH
		model.out=n
			
		// Traitement de l'échantillonnage
		//----------------------------------	
		evtout = []
		evtin = []				
		if Sampling~=0 then
						// Un port d'entrée d'activation
						//------------------------------
						Sampling=1;					
						evtin = 1 							
		end
            
        graphics.exprs=exprs;
		in=[1,1]; intyp=1
		out=[n,1]; outtyp=1
			
		[model,graphics,ok] = set_io(model, graphics, list(in,intyp), list(out, outtyp), evtin, evtout)
			
        model.rpar=[out_high,out_low]'
		model.opar=list(high_lim,low_lim)
        sciblk.graphics=graphics;
		sciblk.model=model
		   
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=RELAIS_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";")	
		   
		
    case "define" then
	
        rpar=[1;0]
		opar=list(1,-1)
		Sampling=0

        model=scicos_model()
		
        model.sim=list("Relais_HC",4)
        model.in=1; model.intyp=1
        model.out=-1; model.outtyp=1
        model.rpar=rpar
        model.opar=opar

        model.dep_ut=[%t %f]

        exprs=[string(opar(1)); string(opar(2)); string(rpar); string(Sampling)]
        gr_i=[]
        sciblk=standard_define([3 2],model,exprs,gr_i)
		
		style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
					"displayedLabel=RELAIS_H";
					"fillColor=#abcdee";
					"shadow=true"];
					
		sciblk.graphics.style = strcat(style_properties,";")	
				
    end
endfunction
