//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	SinuSync
// Auteur			:	PERRICHON Pierre
// Date				:	07/01/2021
// Version			:	1.0
//
// Sujet			: 	Fonction Sinus avec synchronisation au temps d'amorçage
//
// Paramétrage		:	Instant de départ (s)
//						Amplitude
//						Période
//
// Autres données	:	
//						
//						
// Sorties			:	Sinus synchronisé
//						
//						
//						
// Interface IHM	:	SinuSync.sci
// Code objet		;	SinuSyncC.c 
//
// Programme de test:	SinuSync.zos, GeneBF.zcos, SinusAmorti.zcos, Busy.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=SinuSync(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	typ=[];
	
	Module="SinuSync";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
		
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
									
			labels=["Instant de départ (s)";...
			        "Amplitude";...
					"Période (s)"];
					
			meserrPSinus="La période de la fonction Sinus doit être strictement positive";

			if getlanguage()=="en_US" then 
			
				labels=["Starting time (s)";...
			        "Amplitude";...
					"Period (s))"];
					
				meserrPSinus="The period of the Sine function must be strictly positive";
			
			
			end
					
			
			while %t do
			
				[ok,tSinus,ASinus,PSinus,exprs]=scicos_getvalue("Set SinuSync",labels,...
												   list("vec",1,"vec",1,"vec",1),exprs)
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if PSinus<=0 then DefExp=[DefExp;meserrPSinus]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
			end
				
			graphics.exprs=exprs;
			model.rpar=[tSinus;ASinus;PSinus];	
			sciblk.graphics=graphics;
			sciblk.model=model
					
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=SinuSync"];
			sciblk.graphics.style = strcat(style_properties,";");
									
		
		case "define" then
			tSinus=0			// Instant de départ
			ASinus=0.1			// Ampitude initiale
			PSinus=100			// Période initiale
			
			model=scicos_model()
			model.sim=list("SinuSyncC",4)
			model.in=[];model.in2=[];
			model.out=1;model.out2=[1]
					
			rpar=[tSinus;ASinus;PSinus];	
			exprs = [sci2exp(tSinus);sci2exp(ASinus);sci2exp(PSinus)];

			model.blocktype="c"
			model.dep_ut=[%f %t]

			sciblk=standard_define([3. 2.],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=SinuSync"];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
