//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Step2
// Auteur			:	PERRICHON Pierre
// Date				:	14/06/2021
// Version			:	1.0
//
// Sujet			: 	Double échelon de durée programmable
//
// Paramétrage		:	Instant de départ (s)
//						Durée (s)
//						Valeur initiale à t=0
//						Amplitude
//
// Autres données	:	
//						
//						
// Sorties			:	Echelons appliqués au signal d'entrée
//						
//						
//						
// Interface IHM	:	Step2.sci
// Code objet		;	Step2C.c 
//
// Programme de test:	Step2.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Step2(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	typ=[];
	
	Module="Step2";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
		
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end			
						
			labels=["Instant de départ (s)";...
					"Durée (s) ";...
			        "Valeur initiale";...
					"Amplitude"];
					
			if getlanguage()=="en_US" then

				labels=["Starting time (s)";...
					"Duration (s) ";...
			        "Initial value";...
					"Amplitude"];
			
			
			end
					
			
			while %t do
			
				[ok,StartTime,Duree,ValIni,Echelon,exprs]=scicos_getvalue("Set STEP2",labels,...
												   list("vec",1,"vec",1,"vec",1,"vec",1),exprs)
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]

							break; 
									
							SegPalError(1, Module, DefExp);
				end				
			
			end
				
			graphics.exprs=exprs;
			model.rpar=[StartTime;Duree;ValIni;Echelon];	
			sciblk.graphics=graphics;
			sciblk.model=model
					
			style_properties = ["blockWithLabel";
							"verticalLabelPosition=middle";
							"verticalAlign=left";
							"fillColor=#abcdee";
							"fontColor=blue";
							"displayedLabel=Step2"];
			sciblk.graphics.style = strcat(style_properties,";");													
		
		case "define" then
			StartTime=0			// Instant de départ (s)
			Duree=5				// Durée (s)
			ValIni=0			// Valeur initiale
			Echelon=1			// Valeur Finale
			
			model=scicos_model()
			model.sim=list("Step2C",4)
			model.in=[]
			model.out=1
			model.state = ValIni;			
			rpar=[StartTime;;Duree;ValIni;Echelon];	
			exprs = [sci2exp(StartTime);sci2exp(Duree);sci2exp(ValIni);sci2exp(Echelon)];

			model.blocktype="c"
			model.dep_ut=[%f %t]

			sciblk=standard_define([2. 2.],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=Step2"];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
