//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	TempoTrig
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Temporisation retriggerable
//
// Paramétrage		:	Valeur de la tempoeisation (s)
//						
//						
//						
//
// Autres données	:	1 port d'activation en entrée
//						Il et donc prévu pour être positionné derrière un relais (HYSERESIS Xcos)
//						
// Sorties			:	Armement de la tempo si valeur d'entrée nulle, puis décroissance à 0 suivant temps programmé
//						La tempo est réarmée sur chaque clock de l'entrée evint, ou chaque que le port d'entrée régulier a une valeur non nulle
//						
//						
// Interface IHM	:	TempoTrig.sci
// Code objet		;	TempoTrigC.c 
//
// Programme de test:	TempoTrig.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=TempoTrig(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	typ=[];
	
	Module="TempoTrig";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
		
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end			
			
			labels=["Valeur de la temporisation (s) "]
			
			meserrTempo="La valeur de temporisation doit être strictement positive"
			
			if getlanguage()=="en_US" then			
					labels=["Timeout value (s) "]
			
					meserrTempo="The timeout value must be strictly positive"
			end							

			while %t do
			
				[ok,Tempo,exprs]=scicos_getvalue("Set TempoTrig block parameters",..
				labels,list("vec",-1),exprs)
				
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tempo<=0 then DefExp=[DefExp;meserrTempo]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
				
			end
				
			graphics.exprs=exprs;
			model.rpar=[Tempo]
			sciblk.graphics=graphics;
			sciblk.model=model

			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=Tempo Trig. <br>" + string(exprs(1)) + " (s)"];
			sciblk.graphics.style = strcat(style_properties,";");

		
		case "define" then
			Tempo=60			// Valeur d'initialisation par défaut de la temporisation	
			x0=0;				// Valeur initiale nulle de l'intégrateur			
			
			model=scicos_model()
			model.sim=list("TempoTrigC",4)
			model.intyp=1
			model.in=1
			model.in2=1
			model.outtyp=1
			model.out=1
			model.out2=1
			model.evtin=1;			
			model.state = x0;
			model.rpar=[Tempo]
			model.blocktype="c" 
			model.dep_ut=[%f %t]

			exprs=[string(Tempo)]

			sciblk=standard_define([4 3],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=left";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=Tempo Trig. <br>" + string(Tempo) + " (s)"];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
