//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Time_Delay
// Auteur			:	PERRICHON Pierre
// Date				:	12/12/2014
// Version			:	1.0
//
// Sujet			: 	Sortie retardée su signal d'entrée
//						Ce composant est similaire au composant standard TIME_DELAY (Continuous fix delay) de XCOS
//
// Paramétrage		:	Retard pur
//						Valeur initiale du retard à t=0
//						Taille mémoire pour l'allocation du bloc retard (1024 par défaut)
//
// Autres données	:	Le code objet est celui du composant TIME_DELAY (Continuous fix delay) dans la Scilab
//						
//						
// Sorties			:	Signal d'entrée retardé
//						
//						
//						
// Interface IHM	:	Time_Delay.sci
// Code objet		;	SCI/modules/scicos_blocks/src/c/time_delay.c (Type 4) 
//
// Programme de test:	Time_Delay.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=Time_Delay(job, sciblk, e1)
    
    s1= []; s2 = []; // arguments requis mais non utilisés
	
	Module="Time_Delay(";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
    
    select job
	
		case "set" then
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end			
			
			labels=["Retard pur (s)";"Valeur initiale";"Taille de la mémoire tampon"]
			
			if getlanguage()=="en_US" then		
					labels=["Pure delay (s)";"Initial value";"Buffer size"]			
			end			
			
			meserrBuffer="Buffer must be larger than 2";
			meserrDelay="Delay must be strictly positive";

			while %t do
				[ok,T,init,N,exprs]=scicos_getvalue("Set delay parameters",..
				labels, list("vec",1,"vec",1,"vec",1), exprs);
				
				if ~ok then return; end

				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if N<2 then DefExp=[DefExp;meserrBuffer]; end
							if T<=0 then DefExp=[DefExp;meserrDelay]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
								
			end

			[model,graphics,ok]=check_io(model,graphics,[-1],-1,[],[])
				
			graphics.exprs=exprs;
			model.rpar=[T;init];
			model.ipar=N

			model.dep_ut=[%f %t];
					
			sciblk.graphics=graphics;
			sciblk.model=model
					
			// Affichage du retard pur
			//------------------------
			Delai="Time Delay <br>" + " <br>" +  " t=" + string(exprs(1)) + "  (s) <br>  Vi=" + string(exprs(2));

			style_properties = ["blockWithLabel";
						"verticalLabelPosition=middle";
						"verticalAlign=top";
						"fontColor=blue";
						"fontSize=11";
						"displayedLabel= " + Delai;
						"fillColor=#abcdee";
						"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
		
		case "define" then
			nin=1
			T=1
			init=0
			N=1024
			model=scicos_model()
			
			model.sim=list("time_delay",4)	// Bibliothèque Scilab
			
			model.in=[nin]
			model.out=nin
			model.rpar=[T,init];
			model.ipar=N
			model.blocktype="x"
			model.dep_ut=[%f %t]

			exprs=[string(T);string(init);string(N)];
			
			sciblk = standard_define([4 4], model, exprs);

			// Affichage du retard pur
			//------------------------
			Delai="Time Delay <br>" + " <br>" +  " t=" + string(exprs(1)) + "  (s) <br>  Vi=" + string(exprs(2));

			style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=11";
                    "displayedLabel= " + Delai;
					"fillColor=#abcdee";
					"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
						
    end
	
endfunction
