//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	DiracSync
// Auteur			:	PERRICHON Pierre
// Date				:	07/01/2021
// Version			:	1.0
//
// Sujet			: 	Génération de peigne de Dirac synchronisé sur une date d'amorçage
//
// Paramétrage		:	Instant de départ (s)
//						Amplitude
//						Période
//						Durée de l'impulstion (mettre typiquement 100 ms)
//						Impulsion sysmétrique (1) ou non (0) à chaue période
//
// Autres données	:	
//						
//						
// Sorties			:	Peigne de Dirac
//						
//						
//						
// Interface IHM	:	DiracSync.sci
// Code objet		;	DiracSyncC.c 
//
// Programme de test:	DiracSync.zos, GeneBF.zcos
//
//------------------------------------------------------------------------------------------------------------------------
#include <math.h>
#include "scicos_block4.h"
#include "scicos.h"

#include "version.h"
#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif

/*--------------------------------------------------------------------------*/

void DiracSyncC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="DiracSyncC";
	
	double *Out;
	double tDirac, ADirac, PDirac, DDirac, SDirac, difTime;
	long int idifTime,iPDirac,iTime;
	double t;
	
	long int *work = NULL;	
	int flipflop;
			
	switch (flag) {
		
		case Initialization:
					// Allocation d'une zône de travail de 1 doubles
					//----------------------------------------------
					if ((GetWorkPtrs(Cblk) = (double *)MALLOC(2*sizeof(long int)))== NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax");
						return;
					}					
					work = (double *)GetWorkPtrs(Cblk);
					
					work[0] = 0;		// Valeur d'initialisation de la mémoire
					work[1] = -1;		// Valeur d'initialisation de la mémoire

					break;	
		
								
		case ReInitialization:
		case OutputUpdate:
		
					tDirac=Cblk->rpar[0];
					ADirac=Cblk->rpar[1];
					PDirac=Cblk->rpar[2];
					DDirac=Cblk->rpar[3];
					SDirac=Cblk->rpar[4];
					
					Out = GetRealOutPortPtrs(Cblk,1);
					
					t = get_scicos_time();				// t en unité seconde				
					
					if (t<tDirac) {
									Out[0]=0;
									break;
					}
					
					work = (double *)GetWorkPtrs(Cblk);
					

					t=t-tDirac;
					
					iTime=(double) t*1000.;				// Précision ms
					iPDirac=(double) PDirac*1000.;
					
					idifTime=iTime % iPDirac;
					difTime=(double) idifTime/1000.;	// Seconde
					 
					// Evaluation de la fonction F
					//----------------------------
					
					if (difTime<DDirac) {
						
							if (work[0]!=work[1])  {
													// Comptage des fronts montants
													//-----------------------------
													work[0]++;
													work[1]=work[0];
							}
							
							flipflop= SDirac==1 ? work[0] % 2 : 1;
							
							Out[0]= flipflop==0 ? -ADirac : ADirac;							
						
					}
					else {
							Out[0]=0;
							work[1]=0;
					}

					break;	

		case Ending:
					FREE(GetWorkPtrs(Cblk));
					break;				

	}
}