//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	GainVar
// Auteur			:	PERRICHON Pierre
// Date				:	07/07/2017
// Version			:	1.0
//
// Sujet			: 	Répartiteur de jet - Rampe de ralliement lors des commutations de jets d'ue machine PELTON multi-jets
//						
//
// Paramétrage		:	Temps de commutation (s)
//						Valeur initiale du gain variable à t=0
//
// Autres données	:	Le gain objectif doit être supérieur ou égal à 1
//						Un gain nul provoque une erreur de synchronisation sur Xcos !
//						
//						
// Sorties			:	Rampe sur le gain variable
//						
//						
//						
// Interface IHM	:	GainVar.sci
// Code objet		;	GainVarC.c 
//
// Programme de test:	GainVar.zos
//
//------------------------------------------------------------------------------------------------------------------------

#include <math.h>
#include <stdio.h>
#include "scicos.h"
#include "scicos_block4.h"

#include "version.h"

#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif

#include <sciprint.h>

static double LimiteurJet(double x, double sens, double LimIn)
{
double xL;

		xL=x;

		if(sens==0.) return LimIn;
		
		if(sens>0. && xL>=LimIn) return LimIn;
		
		if (sens<0. && xL<=LimIn) return LimIn;	
		
		return xL;		
}



void  GainVarC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="GainVar_VC";
	
	char MesErr[120];
	
	int i = 0;
	
	int numport;
	
	double TauInv;
	
	double *work = NULL;

	int n = GetInPortRows(Cblk,1); 
	
	double *in = GetRealInPortPtrs(Cblk,1);
	double *out = GetRealOutPortPtrs(Cblk,1);
	double *out2 = GetRealOutPortPtrs(Cblk,2);
	double *rpar=GetRparPtrs(Cblk);
	double *xd = GetDerState(Cblk);	
	double *x = GetState(Cblk);

	//double *work = (double *)GetWorkPtrs(Cblk)	
	
	
	switch(flag) {
		
		case Initialization:
					// Vérification des dimensions
					//-----------------------------
					
					
					
					
					
					// Allocation d'une zône de travail de 2*n doubles
					//-------------------------------------------------
					if ((GetWorkPtrs(Cblk) = (double *)MALLOC(2*n*sizeof(double)))== NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax");
						return;
					}					
					work = (double *)GetWorkPtrs(Cblk);
					
					for (i=0; i<2*n; i++) work[i] = 0.0;
					
					for (i=0; i<n; i++) work[2*i+1]=x[i];		// On forme la mémoire d'entrée sur la sortie à l'initialisation

					// sciprint("x0=%f Tau=%f\n",x[0], rpar[0]);

					break;	
  
		case DerivativeState:
					// Evaluation de la fonction C 
					//----------------------------
					TauInv=rpar[0];
		
					work = (double *)GetWorkPtrs(Cblk);
		
					for(i=0; i<n; ++i) xd[i] = work[2*i]*TauInv;;
					
					break;
		
		case OutputUpdate:
		case ReInitialization: 
					// Evaluation de la fonction F
					//----------------------------
		
					work = (double *)GetWorkPtrs(Cblk);
										
					for(i=0; i<n; ++i) {
						
							if (in[i]!=work[2*i+1]) {	
											// Détection d'un changement détat sur l'entrée in[i]
											//---------------------------------------------------
											work[2*i]=(in[i]-x[i]);		// Pente de ralliement
											work[2*i+1]=in[i];			// Mémorisation de l'objetif à atteindre
							}
							
							x[i] = LimiteurJet(x[i],work[2*i],work[2*i+1]); 	// Limitation de la matrice d'état
							
							if(fabs(x[i])<1.e-4) out[i]=0.;				// Insensibilité sur la sortie
							 else out[i]=x[i];
							 
							// Etat de la sortie out2[i]
							//--------------------------
							out2[i]=(in[i]!=0.); 
					}
					
					break;
		
		case Ending:
				FREE(GetWorkPtrs(Cblk));
				break;
	} 
}