//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Interpln_H
// Auteur			:	PERRICHON Pierre
// Date				:	02/09/2021
// Version			:	1.0
//
// Sujet			: 	Interplolation linéaire vectorisée sur une loi XY à absisses X croissantes
//					
//
// Paramétrage		:	in :  size(-1,1) type 1
//						out :  size(-1,1) type 1, idem in
//						Ci-dessous X et Y sont des matrices lignes de dimension mx1
//
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	
//						
//						
// Interface IHM	:	Interpln_H.sci
// Code objet		;	Interpln_HC.c 
//
// Programme de test:	Interpln_H.zos
//
//------------------------------------------------------------------------------------------------------------------------

#include <math.h>
#include <stdio.h>
#include <sciprint.h>
#include "scicos_block4.h"


//------------------------------------------------------------------------------------------------------------------------
// Recherche dichotomique d'un indice d'une valeur val dans une table X classée par abscisse croissante
// Sortie : X[indice]<= val <=X[indice+1]
//------------------------------------------------------------------------------------------------------------------------
int Dichotomie(double val, double *X, int maxpoint)
{
int low=0, mid, up=maxpoint-1;

   while(low<=up) {
	      mid=(low+up)/2;
	      if(val<X[mid]) up=mid-1;
			else if(val==X[mid]) break;
				else low=mid+1;
   }
 
   if(up<0) up=0;
   if(up<low) mid=up;
   if(mid>=maxpoint-1) mid--;
   
	return(mid);
}

//------------------------------------------------------------------------------------------------------------------------
void Interpln_HC(scicos_block *Cblk, int flag)
{
	
static char *Module="Interpln_HC";	
	
int i, k, sizeX, sizeX2, sizeIn;
double *in, *out, *X, *Y, val, a, b;

	switch (flag) {
						
		case ReInitialization:
		case OutputUpdate:	
						sizeIn=GetInPortRows(Cblk,1);
						
						sizeX=GetOparSize(Cblk,1,1); 	// Si X vecteur ligne	
						sizeX2=GetOparSize(Cblk,1,2); 	// Si X vecteur colonne
						
						//sizeX = sizeX>sizeX2 ? sizeX : sizeX2;
						sizeX = sizeX*sizeX2;
						
						in = GetRealInPortPtrs(Cblk,1);
						out = GetRealOutPortPtrs(Cblk,1);

						X=GetOparPtrs(Cblk,1);
						Y=GetOparPtrs(Cblk,2);	

						/*if (flag==ReInitialization) {
							sciprint("sizeX=%i sizeX2=%i\n", sizeX, sizeX2);
							sciprint("X= %f %f %f %f %f\n", X[0], X[1],X[2],X[3],X[4]);
							sciprint("Y= %f %f %f %f %f\n", Y[0], Y[1],Y[2],Y[3],Y[4]);
						}*/
						
						for (i=0;i<sizeIn; i++) {
												val=in[i];
												
												k=Dichotomie(val, X, sizeX);
												
												a=(Y[k+1]-Y[k])/(X[k+1]-X[k]);
												b=Y[k]-a*X[k];
																							
												out[i]=a*val + b;
						}
						
						break;
	}
}

