//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Satur
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Saturations externes
//
// Paramétrage		:	
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	Saturations appliquées au signal d'entrée
//						
//						
//						
// Interface IHM	:	Satur.sci
// Code objet		;	SaturC.c 
//
// Programme de test:	Satur.zos
//
//------------------------------------------------------------------------------------------------------------------------

#include "Common.h"

void  SaturC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="SaturC";
	static int sature=1;
	
	char MesErr[120];
	char *Label;
	int NumBlockErr;
	
	int nmport;
	int i, numport;
	
	int n = GetInPortRows(Cblk,1); 			// Dimension de l'entrée in
	int n_UL = GetInPortRows(Cblk,2); 		// Dimension UL
	int n_LL = GetInPortRows(Cblk,3); 		// Dimension LL
	
	int i_UL=0, i_LL=0;						// Indices UL et LL
	int k_UL, k_LL;						// Incrémenents UL et LL
	
	double *in = GetRealInPortPtrs(Cblk,1);
	double *UL = GetRealInPortPtrs(Cblk,2);
	double *LL = GetRealInPortPtrs(Cblk,3);
	double *out = GetRealOutPortPtrs(Cblk,1);
	
	double data;
	
	k_UL = n_UL>1 ? 1 : 0;
	k_LL = n_LL>1 ? 1 : 0;

	switch (flag) {
		
		case ReInitialization:
				if (n_UL>1 && n_UL!=n) {
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nErreur de dimensionnement port Max (n=%i n_Max=%i)\n",Module,NumBlockErr,n,n_UL);
						Coserror(MesErr);
				}
		
				if (n_LL>1 && n_LL!=n) {
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nErreur de dimensionnement port Min (n=%i n_Min=%i)\n",Module,NumBlockErr,n,n_LL);
						Coserror(MesErr);
				}
				
		case OutputUpdate:
			for(i=0; i<n; ++i) {

					//numport=2;
				
					if (LL[i_LL]>UL[i_UL]) {
						//set_block_error(-1); 	// Données d'entrée inappropriées : voir help c_utils
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nLower limiter LL=%f greater than the Upper UL=%f",Module,NumBlockErr,LL[i],UL[i]);
						Coserror(MesErr);
					}
			
					data=Limiteur(in[i],sature,UL[i_UL],LL[i_LL]);
					out[i]=data;
					
					i_UL+=k_UL;
					i_LL+=k_LL;
			}
			break;
	}
	
}