//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	TempoTrig
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Temporisation retriggerable
//
// Paramétrage		:	Valeur de la tempoeisation (s)
//						
//						
//						
//
// Autres données	:	1 port d'activation en entrée
//						Il et donc prévu pour être positionné derrière un relais (HYSERESIS Xcos)
//						
// Sorties			:	Armement de la tempo si valeur d'entrée nulle, puis décroissance à 0 suivant temps programmé
//						La tempo est réarmée sur chaque clock de l'entrée evint, ou chaque que le port d'entrée régulier a une valeur non nulle
//						
//						
// Interface IHM	:	TempoTrig.sci
// Code objet		;	TempoTrigC.c 
//
// Programme de test:	TempoTrig.zos
//
//------------------------------------------------------------------------------------------------------------------------
#include <math.h>
#include <stdio.h>
#include "scicos_block4.h"


void  TempoTrigC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="TempoTrigC";
	
	double *in,*out,*xd,*x;
			
	switch (flag) {
					
		case DerivativeState : 
					// Evaluation de la fonction C 
					//----------------------------
					xd = GetDerState(Cblk);
					double TauInv=-1./Cblk->rpar[0];		// Pente négative
					
					xd[0] = TauInv;
					break;
			
		case ReInitialization:
		case OutputUpdate:
					// Evaluation de la fonction F
					//----------------------------
					x = GetState(Cblk);
					in = GetRealInPortPtrs(Cblk,1);  					
					out = GetRealOutPortPtrs(Cblk,1);

					if(in[0]!=0.) x[0]=1.;			// Maintien de la temporisation si entrée non nulle
					if(x[0]<0.) x[0]=0.;			// Limiter l'intégrateur à 0., la tempo est écoulée
					out[0]=x[0];
					break;			
	
		case StateUpdate:
					x = GetState(Cblk);				// Réarmement de la tempo sur un clock d'entrée evint
					x[0] = 1.;
					break;
												
	}
}